//	String Helper function
//	(c) Copyright 1990/1998 Marco Savegnago IW3FQG

#include	"strhlp.h"
#include	<string.h>
#include <ctype.h>

// Strip final \n from string.

char * strip_crlf( char *p )
{
	register int i;

	for( i = strlen ( p ) - 1; isspace ( *( p + i ) ) && i >= 0; i-- )
	;

	*( p + i + 1 ) = EOS;

	return p;
}

char * changeSpaceToUnderScore( char *p )
{
	register char *t=p;

	while (*t)
	{
		if (*t == ' ')
			*t = '_';
		t++;
	}

	return p;
}

// 	Return true if the string contain a wildcard

bool isWildCardIncluded(const char* s)
{
	return bool ( strpbrk ( s, "*@#+?<>$\"()") );
}

/*
	This function compare a string with a pattern (taken from fqgbb)

	The pattern can contain this wildcard:

   Special characters in the pattern are:

     * -- Matches 0 or more characters of any type
     @ -- a..z, A..Z
     # -- 0..9
	  + -- A..z, A..Z, 0..9
     ? -- A-F, a-f, 0-9 (a hex number)
     < -- a..z
     > -- A..Z
     = -- Any one character
     " -- Escape.  The next character in the pattern much match exactly.
			 wild cards will not be interpreted.
     ( -- Group.  A "(" starts a group and a ")" ends it.  For each
          group, there must be a matching character in the string.
			 Example: (abc) matches b.  Wild cards are not interpreted.
     \ -- The string to the right of the \ is optional.  It can be
          present and if so must match.  If it is missing, it is
          a match.  Example: a\a will match with a or aa but not ab

Examples:

  95*      --  Will match anything starting with 95 such as 95020,
               95123, 95asb, and 95.
  95===    --  Will match anything that starts with 95 and is 5 characters
               long.  95020 will match but 9502 will not.
  95###    --  Will match anything that starts with 95 and has 5 digits.
               95020 will match but 95ABC will not.
  (WK)6*   --  Will match anything starting with W6 or K6.
  #SOCAL\.CA\.USA\.NA -- Will match with #SOCAL.CA or #SOCAL but not with
                         #SOCAL.XXXX
*/

bool stringMatch( const char *string, const char *pattern, bool caseSensitive, bool usePattern)
{
	register const char *f;
	register const char *p;

	bool match = true;
	bool skip = false;

	if ( usePattern && isWildCardIncluded( pattern ) )
	{
		p = pattern;
		f = string;

		while( ! skip )
		{
			switch ( *p )
			{
         	case	'\0':
            	skip=true;

					if(*f)
               	match=false;
               break;

         	case '\\':
            {
            	while(*p && strlen(p) > 1)
					{
               	if(stringMatch( f, ++p, caseSensitive, usePattern))
                  	return	match = true;

                  while(*p && *p != '\\')
                  	p++;
               }
               break;
            }

				case '*':
            	if(strlen(p) > 1)
               {
               	p++;

                  bool	lmatch = false;

               	while(*(++f))
                  {
							if(*p == *f)
                     {
                     	lmatch	=	true;
                        break;
                     }
                  }

                  if(lmatch && *f && stringMatch( f, p, caseSensitive, usePattern))
                     return	match = true;
               }
               else
                  return	match = true;

					break;
				case '@':
					if( ! isalpha( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '#':
					if( ! isdigit ( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '+':
					if( ! isalnum ( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '?':
            	if( ! isxdigit( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '<':
					if( ! islower( *f ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
				case '>':
					if( ! isupper( *f) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;

				case '(':
				{
					bool lmatch=false;

					while (*++p != ')' )
					{
						if( toupper( *f ) == toupper( *p ) )
									lmatch = true;
					}

					if (!lmatch)
					{
						match=false;
						skip=true;
					}
					p++; f++;
					break;
				}

				case '=':
            	p++;

               if(*f)
						f++;
               else
               {
               	match=false;
						skip = true;
               }

					break;

				case '"':
					p++;
				default:
					if( toupper ( *f ) != toupper( *p ) )
					{
						match = false;
						skip = true;
					}
					p++; f++;
					break;
			}
		}
	}
	else
	{
   	if(caseSensitive)
    	{
			if ( strcmp  ( string, pattern) )
								match = false;
		}
      else
		{
			if ( stricmp ( string, pattern) )
								match = false;
      }
	}

	return ( match );
}

void skipSpace(const char*& s)
{
	while ( isspace ( *s ) && *s++ )
	;
}

void skipSpace(char*& s)
{
	while ( isspace ( *s ) && *s++ )
	;
}

bool isComment(char *s)
{
	return bool ( *s == '!' || *s == ';' || *s == '\n' ||\
								(*s == '/' && *(s+1) == '/' ) );
}

char *striinc(const char *str1, char *str2)
{
	register int max;
	char *p;

	max=strlen(str1);

	for(p=str2;*p;p++)
	{
		if(!strnicmp(str1,p,max))
		  return(p);
	}

	return(NULL);                       /* string1 not found in string2 */
}

char*	GetDelimitedString(char* pszSource, char* pszDestination, word wMaxLen, char cBegDelimiter, char cEndDelimiter, char cEscape)
{
	char*	ps				=	pszSource;
	char*	pd					= 	pszDestination;
	char	cwaitForDelimiter = ' ';

	while(*ps && isspace(*ps) && *ps!=cBegDelimiter)
		ps++;

	if(*ps==NULL)
		return NULL;

	if(*ps==cBegDelimiter)
	{
		ps++;
		cwaitForDelimiter = cEndDelimiter;
	}
	while(*ps && word(pd-pszDestination) < wMaxLen)
	{
		if(*ps==cEscape)
		{
			if(*(ps+1))
			{
				*pd=*(ps+1);
				pd++;
				ps+=2;
			}
			else
				break;
		}
		else if(*ps ==  cwaitForDelimiter)
		{
      	ps++;
			break;
		}
		else
			*pd++=*ps++;
	}

	*pd=NULL;

	return ps;
}

char *strltrim(char *str)
{
    char *p,*q;

    p=q=str;

    while(isspace(*p))
    	p++;

    while(*p)
    	*q++=*p++;

    *q='\0';

    return(str);
}

char *strrtrim(char *str)
{
    register int i;

    for(i=strlen(str)-1;isspace(str[i])&&i>=0;i--) ;
    str[i+1]='\0';

    return(str);
}

//  HexString to int without use assembly ...

int htoi (char *p)
{
	int 	i;
   int 	ret=0;
   int	mult=1;
   int	ch;

   i=strlen(p);

   while(i--)
   {
   	ch = toupper(*(p+i));

      if(isalpha(ch))
      	ch -= ('A' - 10);
      else
      	ch -= '0';

      ret += ch * mult;
      mult *= 16;
	}

   return (ret);
}

char* _2dec(char *b, unsigned long val)
{
	do
	{
   	*b++ = char(((int) (val % 10)) + '0');
      val /= 10;
   }
   while( val );

    *b = '\0';

    strrev(b);

    return b;
}

// octal conversion

char* _2oct(char *b, unsigned long val)
{
    do
	{
        *b++ = char((val & 7) + '0');
        val /= 8;   // let compiler make this a shift if appropriate
    } while( val );

    *b = '\0';

    strrev(b);

	 return b;
}

char *_2hex(char *b, unsigned long val, int upper)
{
   static char digits[2][16] =
	{
    	{'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'},
   	{'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'}
   };

   char *d = upper ? &digits[1][0] : &digits[0][0];

   do
	{
    	*b++ = d[(int)val & 0xf];
		val /= 16;
	} while( val );

   *b = '\0';

	strrev(b);

   return b;
}

char *stripSSID(char *callsign)
{
   char* p=strpbrk(callsign, "- ");

   if(p)
		*p=EOS;

	return callsign;
}

void ExtractWord(char *wd, char *& cp, char *allowed)
{
	while (*cp)
	{
		if (isalnum(*cp) || (allowed!=NULL && strchr(allowed, *cp)))
			*wd++ = *cp++;
		else
			break;
	}

	*wd = '\0';
}
